@@ -6,7 +6,7 @@ from setuptools import setup
 
 setup(
     name="mantarray_magnet_finding",
-    version="0.3.1",
+    version="0.4.0",
     description="Magnet Finding",
     url="https://github.com/CuriBio/mantarray-magnet-finding",
     author="Curi Bio",
@@ -16,9 +16,9 @@ setup(
     package_dir={"": "src"},
     install_requires=[
         "nptyping==1.4.4",
-        "numpy>=1.22.4",  # Tanner (12/3/21): pinned for numba compatibility
-        "scipy>=1.8.1",
-        "numba==0.55.2",
+        "numpy==1.23.4",  # Tanner (12/3/21): pinned for numba compatibility
+        "scipy==1.9.3",
+        "numba==0.56.4",
         "stdlib_utils>=0.4.4",
         "labware-domain-models>=0.3.1",
         "h5py>=3.7.0",
