@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 """Constants for the Mantarray File Manager."""
-from typing import Dict
 import uuid
 
 from immutabledict import immutabledict
 from labware_domain_models import LabwareDefinition
+from mantarray_magnet_finding.constants import NUM_CHANNELS as NUM_CHANNELS_PER_WELL
 
 try:
     from importlib import metadata
@@ -12,21 +12,19 @@ except ImportError:  # pragma: no cover
     import importlib_metadata as metadata  # type: ignore
 PACKAGE_VERSION = metadata.version("pulse3D")
 
-CURI_BIO_ACCOUNT_UUID = uuid.UUID("73f52be0-368c-42d8-a1fd-660d49ba5604")
-CURI_BIO_USER_ACCOUNT_ID = uuid.UUID("455b93eb-c78f-4494-9f73-d3291130f126")
-
 TWENTY_FOUR_WELL_PLATE = LabwareDefinition(row_count=4, column_count=6)
 
+
+NUM_CHANNELS_24_WELL_PLATE = 24 * NUM_CHANNELS_PER_WELL
+
 MIN_SUPPORTED_FILE_VERSION = "0.1.1"
 CURRENT_BETA1_HDF5_FILE_FORMAT_VERSION = "0.4.2"
 CURRENT_BETA2_HDF5_FILE_FORMAT_VERSION = "1.0.0"
 FILE_FORMAT_VERSION_METADATA_KEY = "File Format Version"
 FILE_MIGRATION_PATHS = immutabledict({"0.3.1": "0.4.1", "0.4.1": "0.4.2"})
 
-NOT_APPLICABLE_H5_METADATA = uuid.UUID(
-    "59d92e00-99d5-4460-9a28-5a1a0fe9aecf"
-)  # Eli (1/19/21): H5 files can't store the concept of `None` in their metadata, so using this value to denote that a particular piece of metadata is not available (i.e. after migrating to a newer file format version)
-
+# Eli (1/19/21): H5 files can't store the concept of `None` in their metadata, so using this value to denote that a particular piece of metadata is not available (i.e. after migrating to a newer file format version)
+NOT_APPLICABLE_H5_METADATA = uuid.UUID("59d92e00-99d5-4460-9a28-5a1a0fe9aecf")
 
 HARDWARE_TEST_RECORDING_UUID = uuid.UUID("a2e76058-08cd-475d-a55d-31d401c3cb34")
 UTC_BEGINNING_DATA_ACQUISTION_UUID = uuid.UUID("98c67f22-013b-421a-831b-0ea55df4651e")
@@ -66,6 +64,8 @@ TRIMMED_TIME_FROM_ORIGINAL_END_UUID = uuid.UUID("55f6770d-c369-42ce-a437-5ed89c3
 ORIGINAL_FILE_VERSION_UUID = uuid.UUID("cd1b4063-4a87-4a57-bc12-923ff4890844")
 UTC_TIMESTAMP_OF_FILE_VERSION_MIGRATION_UUID = uuid.UUID("399b2148-09d4-418b-a132-e37df2721938")
 FILE_VERSION_PRIOR_TO_MIGRATION_UUID = uuid.UUID("11b4945b-3cf3-4f67-8bee-7abc3c449756")
+TWITCHES_POINT_UP_UUID = uuid.UUID("97f69f56-f1c6-4c50-8590-7332570ed3c5")
+INTERPOLATION_VALUE_UUID = uuid.UUID("466d0131-06b7-4f0f-ba1e-062a771cb280")
 BOOTUP_COUNTER_UUID = uuid.UUID("b9ccc724-a39d-429a-be6d-3fd29be5037d")
 TOTAL_WORKING_HOURS_UUID = uuid.UUID("f8108718-2fa0-40ce-a51a-8478e5edd4b8")
 TAMPER_FLAG_UUID = uuid.UUID("68d0147f-9a84-4423-9c50-228da16ba895")
@@ -131,13 +131,19 @@ METADATA_UUID_DESCRIPTIONS = immutabledict(
         CHANNEL_FIRMWARE_VERSION_UUID: "Firmware Version (Channel Controller)",
         BOOT_FLAGS_UUID: "Hardware/firmware flags present on device bootup",
         INITIAL_MAGNET_FINDING_PARAMS_UUID: "JSON string of the initial magnet finding params that should be used in Pulse3D",
+        # Optical file values
+        TWITCHES_POINT_UP_UUID: "Flag indicating whether or not the twitches in the data point up or not",
+        INTERPOLATION_VALUE_UUID: "Desired value for optical well data interpolation",
     }
 )
 
 DATETIME_STR_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
+MILLI_TO_BASE_CONVERSION = int(1e3)
 CENTIMILLISECONDS_PER_SECOND = int(1e5)
 MICRO_TO_BASE_CONVERSION = int(1e6)
 MICROSECONDS_PER_CENTIMILLISECOND = 10
+
+
 TISSUE_SENSOR_READINGS = "tissue_sensor_readings"
 REFERENCE_SENSOR_READINGS = "reference_sensor_readings"
 STIMULATION_READINGS = "stimulation_readings"
@@ -145,34 +151,8 @@ TIME_INDICES = "time_indices"
 TIME_OFFSETS = "time_offsets"
 
 
-"""
-constants from mantarray_waveform_analysis library
-"""
-MILLI_TO_BASE_CONVERSION = 1000
-
 TWITCH_PERIOD_UUID = uuid.UUID("6e0cd81c-7861-4c49-ba14-87b2739d65fb")
-
-# This is just the reciprocal of twitch period, but is pre-computed to make downstream pipelines
-# simpler. Frequency is reported in Hz
 TWITCH_FREQUENCY_UUID = uuid.UUID("472d0707-ff87-4198-9374-c28900bb216c")
-AMPLITUDE_UUID = uuid.UUID("89cf1105-a015-434f-b527-4169b9400e26")
-AUC_UUID = uuid.UUID("e7b9a6e4-c43d-4e8b-af7e-51742e252030")
-WIDTH_UUID = uuid.UUID("c4c60d55-017a-4783-9600-f19606de26f3")
-WIDTH_VALUE_UUID = uuid.UUID("05041f4e-c77d-42d9-a2ae-8902f912e9ac")
-WIDTH_RISING_COORDS_UUID = uuid.UUID("2a16acb6-4df7-4064-9d47-5d27ea7a98ad")
-WIDTH_FALLING_COORDS_UUID = uuid.UUID("26e5637d-42c9-4060-aa5d-52209b349c84")
-RELAXATION_VELOCITY_UUID = uuid.UUID("0fcc0dc3-f9aa-4f1b-91b3-e5b5924279a9")
-CONTRACTION_VELOCITY_UUID = uuid.UUID("73961e7c-17ec-42b0-b503-a23195ec249c")
-IRREGULARITY_INTERVAL_UUID = uuid.UUID("61046076-66b9-4b8b-bfec-1e00603743c0")
-
-# Kristian 9/15/21
-FRACTION_MAX_UUID = uuid.UUID("8fe142e2-2504-4c9e-b3dc-817b24c7447e")
-
-# Kristian 10/29/21: for contraction to % width, or peak to % relaxation
-TIME_VALUE_UUID = uuid.UUID("32f5ce6b-e311-4434-8a2a-c2b6bbd81ee6")
-RELAXATION_TIME_UUID = uuid.UUID("0ad56cd1-7bcc-4b57-8076-14366d7f3c6a")
-CONTRACTION_TIME_UUID = uuid.UUID("33b5b0a8-f197-46ef-a451-a254e530757b")
-
 AMPLITUDE_UUID = uuid.UUID("89cf1105-a015-434f-b527-4169b9400e26")
 AUC_UUID = uuid.UUID("e7b9a6e4-c43d-4e8b-af7e-51742e252030")
 WIDTH_UUID = uuid.UUID("c4c60d55-017a-4783-9600-f19606de26f3")
@@ -187,12 +167,10 @@ TIME_DIFFERENCE_UUID = uuid.UUID("1363817a-b1fb-468e-9f1c-ec54fce72dfe")
 TIME_VALUE_UUID = uuid.UUID("32f5ce6b-e311-4434-8a2a-c2b6bbd81ee6")
 RELAXATION_TIME_UUID = uuid.UUID("0ad56cd1-7bcc-4b57-8076-14366d7f3c6a")
 CONTRACTION_TIME_UUID = uuid.UUID("33b5b0a8-f197-46ef-a451-a254e530757b")
-
-# Kristian 11/9/21: full contraction or full relaxation metrics
 BASELINE_TO_PEAK_UUID = uuid.UUID("03ce2d30-3580-4129-9913-2fc2e35eddb7")
 PEAK_TO_BASELINE_UUID = uuid.UUID("1ac2589d-4713-41c0-8dd0-1e6c98600e37")
 
-ALL_METRICS = [
+ALL_METRICS = (
     TWITCH_PERIOD_UUID,
     FRACTION_MAX_UUID,
     AMPLITUDE_UUID,
@@ -206,7 +184,14 @@ ALL_METRICS = [
     PEAK_TO_BASELINE_UUID,
     BASELINE_TO_PEAK_UUID,
     CONTRACTION_TIME_UUID,
-]
+)
+
+
+DEFAULT_TWITCH_WIDTHS = (10, 50, 90)
+DEFAULT_BASELINE_WIDTHS = (10, 90)
+DEFAULT_TWITCH_WIDTH_PERCENTS = tuple(range(10, 95, 5))
+DEFAULT_PROMINENCE_FACTORS = (6, 6)
+DEFAULT_WIDTH_FACTORS = (7, 7)
 
 PRIOR_PEAK_INDEX_UUID = uuid.UUID("80df90dc-21f8-4cad-a164-89436909b30a")
 PRIOR_VALLEY_INDEX_UUID = uuid.UUID("72ba9466-c203-41b6-ac30-337b4a17a124")
@@ -218,31 +203,57 @@ BESSEL_LOWPASS_10_UUID = uuid.UUID("7d64cac3-b841-4912-b734-c0cf20a81e7a")
 BESSEL_LOWPASS_30_UUID = uuid.UUID("eee66c75-4dc4-4eb4-8d48-6c608bf28d91")
 BUTTERWORTH_LOWPASS_30_UUID = uuid.UUID("de8d8cef-65bf-4119-ada7-bdecbbaa897a")
 
-# General mangetic field to force conversion factor. Obtained 03/09/2021 by Kevin Gray, Valid as of 11/19/21
+# General magnetic field to force conversion factor. Obtained 03/09/2021 by Kevin Gray
+# Valid as of 11/19/21
 MILLIMETERS_PER_MILLITESLA = 23.25
+# Valid as of 09/30/22
 NEWTONS_PER_MILLIMETER = 0.000159
 
+CALIBRATION_STIFFNESS_FACTOR = 1
+CARDIAC_STIFFNESS_FACTOR = 1
+SKM_STIFFNESS_FACTOR = 12
+ROW_LABEL_TO_VARIABLE_STIFFNESS_FACTOR = immutabledict({"A": 12, "B": 9, "C": 6, "D": 3})
+
+
 # Beta 1 GMR to magnetic field conversion values. Valid as of 11/19/21
 MILLIVOLTS_PER_MILLITESLA = 1073.6  # Obtained 03/09/2021 by Kevin Gray
 MIDSCALE_CODE = 0x800000
-RAW_TO_SIGNED_CONVERSION_VALUE = 2 ** 23  # subtract this value from raw hardware data
+RAW_TO_SIGNED_CONVERSION_VALUE = 2**23  # subtract this value from raw hardware data
 REFERENCE_VOLTAGE = 2.5
 ADC_GAIN = 2
 
 # Beta 2 Memsic to magnetic field conversion factors. Valid as of 11/19/21
-MEMSIC_CENTER_OFFSET = 2 ** 15
-MEMSIC_MSB = 2 ** 16
+MEMSIC_CENTER_OFFSET = 2**15
+MEMSIC_MSB = 2**16
 MEMSIC_FULL_SCALE = 16
 GAUSS_PER_MILLITESLA = 10
 
 
-MIN_NUMBER_PEAKS = 3
-MIN_NUMBER_VALLEYS = 3
+MIN_NUMBER_PEAKS = 1
+MIN_NUMBER_VALLEYS = 1
+
+
+MIN_EXPERIMENT_ID = 0
+MAX_CARDIAC_EXPERIMENT_ID = 99
+MAX_SKM_EXPERIMENT_ID = 199
+MAX_VARIABLE_EXPERIMENT_ID = 299
+MAX_EXPERIMENT_ID = 999
+
+CARDIAC_STIFFNESS_LABEL = "Cardiac (1x)"
+SKM_STIFFNESS_LABEL = "Skeletal Muscle (12x)"
+VARIABLE_STIFFNESS_LABEL = "Variable (Row A: 12x, Row B: 9x, Row C: 6x, Row D: 3x)"
+
+POST_STIFFNESS_LABEL_TO_FACTOR = immutabledict(
+    {
+        CARDIAC_STIFFNESS_LABEL: CARDIAC_STIFFNESS_FACTOR,
+        SKM_STIFFNESS_LABEL: SKM_STIFFNESS_FACTOR,
+        VARIABLE_STIFFNESS_LABEL: ROW_LABEL_TO_VARIABLE_STIFFNESS_FACTOR,
+    }
+)
+
+POST_STIFFNESS_OVERRIDE_OPTIONS = (CARDIAC_STIFFNESS_FACTOR, SKM_STIFFNESS_FACTOR)
 
 
-"""
-pulse3D constants
-"""
 METADATA_EXCEL_SHEET_NAME = "metadata"
 METADATA_RECORDING_ROW_START = 0
 METADATA_INSTRUMENT_ROW_START = METADATA_RECORDING_ROW_START + 4
@@ -258,22 +269,29 @@ TWITCH_FREQUENCIES_CHART_SHEET_NAME = "twitch-frequencies-plots"
 FORCE_FREQUENCY_RELATIONSHIP_SHEET = "force-frequency-relationship"
 
 INTERPOLATED_DATA_PERIOD_SECONDS = 1 / 100
-INTERPOLATED_DATA_PERIOD_US = INTERPOLATED_DATA_PERIOD_SECONDS * MICRO_TO_BASE_CONVERSION
-TSP_TO_DEFAULT_FILTER_UUID = {  # Tissue Sampling Period (µs) to default Pipeline Filter UUID
-    9600: BESSEL_LOWPASS_10_UUID,
-    1600: BUTTERWORTH_LOWPASS_30_UUID,
-}
+INTERPOLATED_DATA_PERIOD_US = int(INTERPOLATED_DATA_PERIOD_SECONDS * MICRO_TO_BASE_CONVERSION)
+# Tissue Sampling Period (µs) to default Pipeline Filter UUID
+TSP_TO_DEFAULT_FILTER_UUID = {9600: BESSEL_LOWPASS_10_UUID, 1600: BUTTERWORTH_LOWPASS_30_UUID}
 
 DEFAULT_CELL_WIDTH = 64
+DEFAULT_CELL_HEIGHT = 20
+
 CHART_ALPHA = 60  # for full/snapshots -- num pixels between left figure edge and plot area
 CHART_GAMMA = 150  # for full/snapshots -- num pixels between right figure edge and plot area
 CHART_PIXELS_PER_SECOND = 35  # for full/snapshots -- number of pixels per second
 CHART_MAXIMUM_SNAPSHOT_LENGTH = 10
-CHART_HEIGHT = 300
+
 CHART_HEIGHT_CELLS = 15
+CHART_HEIGHT = DEFAULT_CELL_HEIGHT * CHART_HEIGHT_CELLS
+
+STIM_CHART_HEIGHT_CELLS = CHART_HEIGHT_CELLS - 2
+STIM_CHART_HEIGHT = DEFAULT_CELL_HEIGHT * STIM_CHART_HEIGHT_CELLS
+
 CHART_FIXED_WIDTH_CELLS = 8
 CHART_FIXED_WIDTH = DEFAULT_CELL_WIDTH * CHART_FIXED_WIDTH_CELLS
+
 PEAK_VALLEY_COLUMN_START = 100
+STIM_DATA_COLUMN_START = 200
 SECONDS_PER_CELL = 2.5
 
 CALCULATED_METRIC_DISPLAY_NAMES = {
@@ -281,15 +299,15 @@ CALCULATED_METRIC_DISPLAY_NAMES = {
     TWITCH_FREQUENCY_UUID: "Twitch Frequency (Hz)",
     AMPLITUDE_UUID: "Active Twitch Force (μN)",
     FRACTION_MAX_UUID: "Fraction of Maximum Active Twitch Force (μN)",
-    AUC_UUID: "Energy (μJ)",
+    AUC_UUID: "Area Under Curve (μN * second)",
     CONTRACTION_VELOCITY_UUID: "Twitch Contraction Velocity (μN/second)",
     RELAXATION_VELOCITY_UUID: "Twitch Relaxation Velocity (μN/second)",
     IRREGULARITY_INTERVAL_UUID: "Twitch Interval Irregularity (seconds)",
     TIME_DIFFERENCE_UUID: "Time Difference (seconds)",
     WIDTH_UUID: "Twitch Width {} (seconds)",
-    RELAXATION_TIME_UUID: "Time From Peak to Relaxation {} (seconds)",
     CONTRACTION_TIME_UUID: "Time From Contraction {} to Peak (seconds)",
     BASELINE_TO_PEAK_UUID: "Time From Contraction {} to Peak (seconds)",
+    RELAXATION_TIME_UUID: "Time From Peak to Relaxation {} (seconds)",
     PEAK_TO_BASELINE_UUID: "Time From Peak to Relaxation {} (seconds)",
 }
 
@@ -311,38 +329,6 @@ CALCULATED_METRICS = immutabledict(
     }
 )
 
-COORDS = (10, 25, 50, 75, 90)
-TWITCH_WIDTH_METRIC_DISPLAY_NAMES: Dict[int, str] = immutabledict(
-    (coord, f"Twitch Width {coord} (seconds)") for coord in reversed(COORDS)
-)
-CONTRACTION_COORDINATES_DISPLAY_NAMES: Dict[int, str] = immutabledict(
-    (coord, f"Contraction Coordinates {coord}") for coord in reversed(COORDS)
-)
-RELAXATION_COORDINATES_DISPLAY_NAMES: Dict[int, str] = immutabledict(
-    (coord, f"Relaxation Coordinates {coord}") for coord in COORDS
-)
-CONTRACTION_TIME_DIFFERENCE_DISPLAY_NAMES: Dict[int, str] = immutabledict(
-    (coord, f"Time From Contraction {coord} to Peak (seconds)") for coord in reversed(COORDS)
-)
-RELAXATION_TIME_DIFFERENCE_DISPLAY_NAMES: Dict[int, str] = immutabledict(
-    (coord, f"Time From Peak to Relaxation {coord} (seconds)") for coord in COORDS
-)
-
-ALL_FORMATS = immutabledict({"CoV": {"num_format": "0.00%"}})
-
-TWITCHES_POINT_UP_UUID = uuid.UUID("97f69f56-f1c6-4c50-8590-7332570ed3c5")
-INTERPOLATION_VALUE_UUID = uuid.UUID("466d0131-06b7-4f0f-ba1e-062a771cb280")
-mutable_metadata_uuid_descriptions = dict(
-    METADATA_UUID_DESCRIPTIONS
-)  # create a mutable version to add in the new values specific to the SDK (.update is an in-place operation that doesn't return the dictionary, so chaining is difficult)
-mutable_metadata_uuid_descriptions.update(
-    {
-        TWITCHES_POINT_UP_UUID: "Flag indicating whether or not the twitches in the data point up or not",
-        INTERPOLATION_VALUE_UUID: "Desired value for optical well data interpolation",
-    }
-)
-METADATA_UUID_DESCRIPTIONS = immutabledict(mutable_metadata_uuid_descriptions)
-
 EXCEL_OPTICAL_METADATA_CELLS = immutabledict(
     {
         WELL_NAME_UUID: "E2",
@@ -356,9 +342,9 @@ EXCEL_OPTICAL_METADATA_CELLS = immutabledict(
 )
 
 
-"""
-Magnet Finding
-"""
-
 # 10 seconds at sampling rate of 100Hz
 BASELINE_MEAN_NUM_DATA_POINTS = 10 * 100
+
+
+MIN_FILE_VERSION_FOR_STIM_INTERPOLATION = "1.3.0"
+STIM_COMPLETE_SUBPROTOCOL_IDX = 255
