@@ -3,11 +3,10 @@ import datetime
 import glob
 import json
 import logging
-import math
 import os
-import copy
 import tempfile
 from typing import Any
+from typing import List
 from typing import Optional
 from typing import Union
 import uuid
@@ -26,9 +25,14 @@ from xlsxwriter.utility import xl_cell_to_rowcol
 
 from .compression_cy import compress_filtered_magnetic_data
 from .constants import *
+from .exceptions import NoRecordingFilesLoadedError
+from .exceptions import SubprotocolFormatIncompatibleWithInterpolationError
 from .magnet_finding import find_magnet_positions
 from .magnet_finding import fix_dropped_samples
 from .magnet_finding import format_well_file_data
+from .stimulation import aggregate_timepoints
+from .stimulation import create_stim_session_waveforms
+from .stimulation import realign_interpolated_stim_data
 from .transforms import apply_empty_plate_calibration
 from .transforms import apply_noise_filtering
 from .transforms import apply_sensitivity_calibration
@@ -37,6 +41,8 @@ from .transforms import calculate_force_from_displacement
 from .transforms import calculate_voltage_from_gmr
 from .transforms import create_filter
 from .transforms import noise_cancellation
+from .utils import get_experiment_id
+from .utils import get_stiffness_factor
 from .utils import truncate
 from .utils import truncate_float
 
@@ -64,9 +70,26 @@ class MantarrayH5FileCreator(h5py.File):
 
 
 class WellFile:
-    def __init__(self, file_path: str, sampling_period=None):
+    def __init__(
+        self,
+        file_path: str,
+        sampling_period: Optional[Union[int, float]] = None,
+        # TODO unit test the stiffness factor (auto and override) and has_inverted_post_magnet?
+        stiffness_factor: Optional[int] = None,
+        has_inverted_post_magnet: bool = False,
+    ):
         self.displacement: NDArray[(2, Any), np.float64]
         self.force: NDArray[(2, Any), np.float64]
+        self.stim_sessions: List[NDArray[(2, Any), int]] = []
+
+        if stiffness_factor not in (*POST_STIFFNESS_OVERRIDE_OPTIONS, None):
+            raise ValueError(
+                f"Invalid Post Stiffness {stiffness_factor}, must be in {POST_STIFFNESS_OVERRIDE_OPTIONS}"
+            )
+
+        self.has_inverted_post_magnet = has_inverted_post_magnet
+
+        self.stiffness_override = stiffness_factor is not None
 
         if file_path.endswith(".h5"):
             with h5py.File(file_path, "r") as h5_file:
@@ -76,26 +99,37 @@ class WellFile:
 
                 self.attrs = {attr: h5_file.attrs[attr] for attr in list(h5_file.attrs)}
                 self.version = self[FILE_FORMAT_VERSION_METADATA_KEY]
+                self.stimulation_protocol = self.get(STIMULATION_PROTOCOL_UUID, "null")
 
-                # extract datetime
-                self[UTC_BEGINNING_RECORDING_UUID] = self._extract_datetime(UTC_BEGINNING_RECORDING_UUID)
-                self[UTC_BEGINNING_DATA_ACQUISTION_UUID] = self._extract_datetime(
-                    UTC_BEGINNING_DATA_ACQUISTION_UUID
+                if self.stiffness_override:
+                    self.stiffness_factor = stiffness_factor
+                elif not self.get(IS_CALIBRATION_FILE_UUID, False):
+                    # earlier versions of files do not have the IS_CALIBRATION_FILE_UUID in their metadata
+                    experiment_id = get_experiment_id(self[PLATE_BARCODE_UUID])
+                    self.stiffness_factor = get_stiffness_factor(experiment_id, self[WELL_INDEX_UUID])
+                else:
+                    # calibration recordings do not have an associated barcode or post stiffness since they
+                    # are creatd when a plate is not even on the istrument, so just set the stiffness factor to 1
+                    self.stiffness_factor = CALIBRATION_STIFFNESS_FACTOR
+
+                # extract datetimes
+                for uuid_ in (
+                    UTC_BEGINNING_RECORDING_UUID,
+                    UTC_BEGINNING_DATA_ACQUISTION_UUID,
+                    UTC_FIRST_TISSUE_DATA_POINT_UUID,
+                ):
+                    self[uuid_] = self._extract_datetime(uuid_)
+
+                self.tissue_sampling_period = (
+                    sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
                 )
-                self[UTC_FIRST_TISSUE_DATA_POINT_UUID] = self._extract_datetime(
-                    UTC_FIRST_TISSUE_DATA_POINT_UUID
-                )
-                if self.version < VersionInfo.parse("1.0.0"):  # Ref data not yet added to these files
+
+                if self.version < VersionInfo.parse("1.0.0"):
+                    # Ref data not yet added to Beta 2 files
                     self[UTC_FIRST_REF_DATA_POINT_UUID] = self._extract_datetime(
                         UTC_FIRST_REF_DATA_POINT_UUID
                     )
-
-                # setup noise filter
-                if self.version < VersionInfo.parse("1.0.0"):
-                    # should probably add beta 2 file support here and remove this condition
-                    self.tissue_sampling_period = (
-                        sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
-                    )
+                    # setup noise filter, should probably add beta 2 file support for this
                     self.noise_filter_uuid = (
                         TSP_TO_DEFAULT_FILTER_UUID[self.tissue_sampling_period]
                         if self.is_magnetic_data
@@ -106,36 +140,32 @@ class WellFile:
                         if self.noise_filter_uuid
                         else None
                     )
-                else:
-                    self.noise_filter_uuid = None
-                    self.filter_coefficients = None
-                    self.tissue_sampling_period = (
-                        sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
-                    )
-                    self.is_magnetic_data = False
 
-                is_untrimmed = self.get(IS_FILE_ORIGINAL_UNTRIMMED_UUID, True)
-                time_trimmed = None if is_untrimmed else self.attrs[TRIMMED_TIME_FROM_ORIGINAL_START_UUID]
-
-                # load sensor data. This is only possible to do for Beta 1 data files
-                if self.version < VersionInfo.parse("1.0.0"):
-                    self[TISSUE_SENSOR_READINGS] = self._load_reading(
-                        h5_file, TISSUE_SENSOR_READINGS, time_trimmed
-                    )
-                    self[REFERENCE_SENSOR_READINGS] = self._load_reading(
-                        h5_file, REFERENCE_SENSOR_READINGS, time_trimmed
-                    )
+                    # load sensor data. This is only possible to do here for Beta 1 data files
+                    self[TISSUE_SENSOR_READINGS] = self._load_reading(h5_file, TISSUE_SENSOR_READINGS)
+                    self[REFERENCE_SENSOR_READINGS] = self._load_reading(h5_file, REFERENCE_SENSOR_READINGS)
                     self._load_magnetic_data()
                 else:
-                    self[TIME_INDICES] = h5_file[TIME_INDICES][:]
-                    self[TIME_OFFSETS] = h5_file[TIME_OFFSETS][:]
-                    self[TISSUE_SENSOR_READINGS] = h5_file[TISSUE_SENSOR_READINGS][:]
-                    self[REFERENCE_SENSOR_READINGS] = h5_file[REFERENCE_SENSOR_READINGS][:]
+                    if self.has_inverted_post_magnet:
+                        raise ValueError("V1 algorithm will automatically fix this issue")
+
+                    self.noise_filter_uuid = None
+                    self.filter_coefficients = None
+                    self.is_magnetic_data = False
+
+                    for dataset in (
+                        TIME_INDICES,
+                        TIME_OFFSETS,
+                        TISSUE_SENSOR_READINGS,
+                        REFERENCE_SENSOR_READINGS,
+                        STIMULATION_READINGS,
+                    ):
+                        self[dataset] = h5_file[dataset][:]
 
         elif file_path.endswith(".xlsx"):
             self._excel_sheet = _get_single_sheet(file_path)
             self.file_name = os.path.basename(file_path)
-            self.attrs = {k: v for (k, v) in _load_optical_file_attrs(self._excel_sheet).items()}
+            self.attrs = {k: v for k, v in _load_optical_file_attrs(self._excel_sheet).items()}
             self.version = self[FILE_FORMAT_VERSION_METADATA_KEY]
             self.is_magnetic_data = False
             self.is_force_data = (
@@ -151,17 +181,26 @@ class WellFile:
                 else None
             )
 
+            self.stiffness_factor = stiffness_factor if stiffness_factor else CARDIAC_STIFFNESS_FACTOR
+
             self._load_magnetic_data()
 
     def _load_magnetic_data(self):
         adj_raw_tissue_reading = self[TISSUE_SENSOR_READINGS].copy()
-        f = MICROSECONDS_PER_CENTIMILLISECOND if self.is_magnetic_data else MICRO_TO_BASE_CONVERSION
-        adj_raw_tissue_reading[0] *= f
+
+        time_conversion = (
+            MICROSECONDS_PER_CENTIMILLISECOND if self.is_magnetic_data else MICRO_TO_BASE_CONVERSION
+        )
+        adj_raw_tissue_reading[0] *= time_conversion
 
         # magnetic data is flipped
         if self.is_magnetic_data:
             adj_raw_tissue_reading[1] *= -1
 
+        # some posts have a flipped magnet
+        if self.has_inverted_post_magnet:
+            adj_raw_tissue_reading[1] *= -1
+
         self.raw_tissue_magnetic_data: NDArray[(2, Any), int] = adj_raw_tissue_reading
         self.raw_reference_magnetic_data: NDArray[(2, Any), int] = self[REFERENCE_SENSOR_READINGS].copy()
 
@@ -174,20 +213,19 @@ class WellFile:
         )
 
         self.noise_cancelled_magnetic_data: NDArray[(2, Any), int] = noise_cancellation(
-            self.sensitivity_calibrated_tissue_gmr,
-            self.sensitivity_calibrated_reference_gmr,
+            self.sensitivity_calibrated_tissue_gmr, self.sensitivity_calibrated_reference_gmr
         )
 
         self.fully_calibrated_magnetic_data: NDArray[(2, Any), int] = apply_empty_plate_calibration(
             self.noise_cancelled_magnetic_data
         )
 
+        self.noise_filtered_magnetic_data: NDArray[(2, Any), int]
         if self.noise_filter_uuid is None:
-            self.noise_filtered_magnetic_data: NDArray[(2, Any), int] = self.fully_calibrated_magnetic_data
+            self.noise_filtered_magnetic_data = self.fully_calibrated_magnetic_data
         else:
-            self.noise_filtered_magnetic_data: NDArray[(2, Any), int] = apply_noise_filtering(
-                self.fully_calibrated_magnetic_data,
-                self.filter_coefficients,
+            self.noise_filtered_magnetic_data = apply_noise_filtering(
+                self.fully_calibrated_magnetic_data, self.filter_coefficients
             )
 
         self.compressed_magnetic_data: NDArray[(2, Any), int] = compress_filtered_magnetic_data(
@@ -200,19 +238,18 @@ class WellFile:
             self.compressed_voltage
         )
         self.compressed_force: NDArray[(2, Any), np.float32] = calculate_force_from_displacement(
-            self.compressed_displacement, in_mm=False
+            self.compressed_displacement, stiffness_factor=self.stiffness_factor, in_mm=False
         )
 
         self.voltage: NDArray[(2, Any), np.float32] = calculate_voltage_from_gmr(
             self.noise_filtered_magnetic_data
         )
-
         self.displacement: NDArray[(2, Any), np.float64] = calculate_displacement_from_voltage(self.voltage)
         self.force: NDArray[(2, Any), np.float64] = calculate_force_from_displacement(
-            self.displacement, in_mm=False
+            self.displacement, stiffness_factor=self.stiffness_factor, in_mm=False
         )
 
-    def get(self, key, default):
+    def get(self, key, default=None):
         try:
             return self[key]
         except Exception:
@@ -267,87 +304,84 @@ class WellFile:
             tzinfo=datetime.timezone.utc
         )
 
-    def _load_reading(self, h5_file, reading_type: str, time_trimmed) -> NDArray[(Any, Any), int]:
-        recording_start_index = self[START_RECORDING_TIME_INDEX_UUID]
-        beginning_data_acquisition_ts = self[UTC_BEGINNING_DATA_ACQUISTION_UUID]
-
-        if reading_type == REFERENCE_SENSOR_READINGS:
-            initial_timestamp = self[UTC_FIRST_REF_DATA_POINT_UUID]
-            sampling_period = self[REF_SAMPLING_PERIOD_UUID]
-        else:
-            initial_timestamp = self[UTC_FIRST_TISSUE_DATA_POINT_UUID]
-            sampling_period = self[TISSUE_SAMPLING_PERIOD_UUID]
-
-        recording_start_index_useconds = int(recording_start_index) * MICROSECONDS_PER_CENTIMILLISECOND
-        timestamp_of_start_index = beginning_data_acquisition_ts + datetime.timedelta(
-            microseconds=recording_start_index_useconds
-        )
-
-        time_delta = initial_timestamp - timestamp_of_start_index
-        time_delta_centimilliseconds = int(
-            time_delta / datetime.timedelta(microseconds=MICROSECONDS_PER_CENTIMILLISECOND)
-        )
-
+    def _load_reading(self, h5_file, reading_type: str) -> NDArray[(Any, Any), int]:
+        sampling_period = self[
+            REF_SAMPLING_PERIOD_UUID
+            if reading_type == REFERENCE_SENSOR_READINGS
+            else TISSUE_SAMPLING_PERIOD_UUID
+        ]
         time_step = int(sampling_period / MICROSECONDS_PER_CENTIMILLISECOND)
 
-        # adding `[:]` loads the data as a numpy array giving us more flexibility of multi-dimensional arrays
-        data = h5_file[reading_type][:]
-        if len(data.shape) == 1:
-            data = data.reshape(1, data.shape[0])
+        tissue_contraction_amplitudes = h5_file[reading_type][:]
+        num_data_points = tissue_contraction_amplitudes.shape[-1]
 
-        # fmt: off
-        # black reformatted this into a very ugly few lines of code
-        times = np.mgrid[: data.shape[1],] * time_step
-        # fmt: on
+        timepoints = np.arange(num_data_points) * time_step
 
-        if time_trimmed:
-            new_times = times + time_delta_centimilliseconds
-            start_index = _find_start_index(time_trimmed, new_times)
-            time_delta_centimilliseconds = int(new_times[start_index])
-
-        return np.concatenate((times + time_delta_centimilliseconds, data), dtype=np.int32)
+        return np.array([timepoints, tissue_contraction_amplitudes], dtype=np.int32)
 
 
 class PlateRecording:
     def __init__(
         self,
         path,
-        use_mean_of_baseline=True,
-        force_df: pd.DataFrame = None,
+        recording_df: pd.DataFrame = None,
         start_time: Union[float, int] = 0,
-        end_time: Union[float, int] = np.inf,
+        end_time: Optional[Union[float, int]] = None,
+        # TODO unit test the stiffness factor (auto and override), inverted_post_magnet_wells
+        stiffness_factor: Optional[int] = None,
+        inverted_post_magnet_wells: Optional[List[str]] = None,
     ):
         self.path = path
         self.wells = []
         self._iter = 0
         self.is_optical_recording = False
-        self.start_time = start_time * MICRO_TO_BASE_CONVERSION
-        self.end_time = end_time * MICRO_TO_BASE_CONVERSION
+
+        # Tanner (11/16/22): due to the needs of the scientists for the full analysis,
+        # these params should only be used in the recording snapshot.
+        # These params also have no effect on Beta 1 data.
+        if start_time < 0:
+            raise ValueError("'start_time' must be >= 0")
+        if end_time and start_time >= end_time:
+            raise ValueError("'start_time' must be < 'end_time'")
+        self.start_time_secs = start_time
+        self.end_time_secs = end_time
+
+        self._created_from_dataframe = recording_df is not None
 
         if self.path.endswith(".zip"):
             with tempfile.TemporaryDirectory() as tmpdir:
                 zf = zipfile.ZipFile(path)
                 zf.extractall(path=tmpdir)
-                self.wells, calibration_recordings = load_files(tmpdir)
+                self.wells, calibration_recordings = load_files(
+                    tmpdir, stiffness_factor, inverted_post_magnet_wells
+                )
         elif self.path.endswith(".xlsx"):  # optical file
             self.is_optical_recording = True
-            well_file = WellFile(self.path)
+            well_file = WellFile(self.path, stiffness_factor=stiffness_factor)
             self.wells = [None] * (well_file[WELL_INDEX_UUID] + 1)
             self.wells[well_file[WELL_INDEX_UUID]] = well_file
         else:  # .h5 files
-            self.wells, calibration_recordings = load_files(self.path)
+            self.wells, calibration_recordings = load_files(
+                self.path, stiffness_factor, inverted_post_magnet_wells
+            )
 
-        if self.wells:  # might not be any well files in the path
-            # currently file versions 1.0.0 and above must have all their data processed together
-            if not self.is_optical_recording and self.wells[0].version >= VersionInfo.parse("1.0.0"):
-                if force_df is None:
-                    self._process_plate_data(
-                        calibration_recordings, use_mean_of_baseline=use_mean_of_baseline
-                    )
-                else:
-                    self._load_dataframe(force_df)
+        # make sure at least one WellFile was loaded
+        if not any(self.wells):
+            raise NoRecordingFilesLoadedError()
 
-    def _process_plate_data(self, calibration_recordings, use_mean_of_baseline):
+        # currently file versions 1.0.0 and above must have all their data processed together
+        if not self.is_optical_recording and self.wells[0].version >= VersionInfo.parse("1.0.0"):
+            if self._created_from_dataframe:
+                self._load_dataframe(recording_df)
+            else:
+                self._process_plate_data(calibration_recordings)
+
+                if self.wells[0][FILE_FORMAT_VERSION_METADATA_KEY] >= VersionInfo.parse(
+                    MIN_FILE_VERSION_FOR_STIM_INTERPOLATION
+                ):
+                    self._process_stim_data()
+
+    def _process_plate_data(self, calibration_recordings):
         if not all(isinstance(well_file, WellFile) for well_file in self.wells) or len(self.wells) != 24:
             raise NotImplementedError("All 24 wells must have a recording file present")
 
@@ -361,27 +395,22 @@ class PlateRecording:
             self.wells[0].get(INITIAL_MAGNET_FINDING_PARAMS_UUID, r"{}")
         )
 
-        # load data
-        plate_data_array = format_well_file_data(self.wells)
+        sampling_period_us = self.wells[0][TISSUE_SAMPLING_PERIOD_UUID]
+        sampling_freq = MICRO_TO_BASE_CONVERSION / sampling_period_us
+        start_idx = int(self.start_time_secs * sampling_freq)
+        end_idx = int(self.end_time_secs * sampling_freq) if self.end_time_secs else None
+        analysis_window = slice(start_idx, end_idx)
+
+        # load tissue data
+        plate_data_array = format_well_file_data(self.wells)[:, analysis_window]
         fixed_plate_data_array = fix_dropped_samples(plate_data_array)
         plate_data_array_mt = calculate_magnetic_flux_density_from_memsic(fixed_plate_data_array)
+        # load 'calibration' data
         baseline_data = format_well_file_data(calibration_recordings)
         baseline_data_mt = calculate_magnetic_flux_density_from_memsic(baseline_data)
 
         # create baseline data array
-        if use_mean_of_baseline:
-            baseline_data_mt = np.mean(
-                baseline_data_mt[:, :, :, -BASELINE_MEAN_NUM_DATA_POINTS:], axis=3
-            ).reshape((24, 3, 3, 1))
-        else:
-            # extend baseline data (if necessary) so that it has at least as many samples as the tissue data
-            num_samples_in_recording = plate_data_array_mt.shape[-1]
-            num_samples_in_baseline = baseline_data_mt.shape[-1]
-            num_times_to_duplicate = math.ceil(num_samples_in_recording / num_samples_in_baseline)
-            # truncate so baseline data has exactly as many samples as the tissue data
-            baseline_data_mt = np.tile(baseline_data_mt, num_times_to_duplicate)[
-                :, :, :, :num_samples_in_recording
-            ]
+        baseline_data_mt = np.mean(baseline_data_mt[:, -BASELINE_MEAN_NUM_DATA_POINTS:], axis=1)
 
         # pass data into magnet finding alg
         log.info("Estimating magnet positions")
@@ -393,36 +422,82 @@ class PlateRecording:
 
         # create displacement and force arrays for each WellFile
         log.info("Create diplacement and force data for each well")
-        for well_idx in range(24):
-            well_file = self.wells[well_idx]
+        for well_idx, well_file in enumerate(self):
             x = estimated_magnet_positions["X"][:, well_idx]
             if flip_data:
                 x *= -1
+
             # have time indices start at 0
-            adjusted_time_indices = well_file[TIME_INDICES] - well_file[TIME_INDICES][0]
-            start_idx, end_idx = truncate(
-                source_series=adjusted_time_indices,
-                lower_bound=self.start_time,
-                upper_bound=self.end_time,
+            time_indices = well_file[TIME_INDICES]
+            adjusted_time_indices = time_indices[analysis_window] - time_indices[start_idx]
+
+            well_file.displacement = np.array([adjusted_time_indices, x])
+            well_file.force = calculate_force_from_displacement(
+                well_file.displacement, stiffness_factor=well_file.stiffness_factor
             )
 
-            well_file.displacement = np.array(
-                [adjusted_time_indices[start_idx : end_idx + 1], x[start_idx : end_idx + 1]]
+    def _process_stim_data(self) -> None:
+        log.info("Interpolating stim sessions")
+
+        start_time_us = int(self.start_time_secs * MICRO_TO_BASE_CONVERSION)
+        end_time_us = (
+            int(self.end_time_secs * MICRO_TO_BASE_CONVERSION)
+            if self.end_time_secs
+            else self.wells[0][TIME_INDICES][-1]
+        )
+
+        for wf in self:
+            if not wf[STIMULATION_READINGS].shape[-1]:
+                continue
+
+            stim_protocol = json.loads(wf[STIMULATION_PROTOCOL_UUID])
+
+            try:
+                stim_sessions_waveforms = create_stim_session_waveforms(
+                    stim_protocol["subprotocols"], wf[STIMULATION_READINGS], start_time_us, end_time_us
+                )
+            except SubprotocolFormatIncompatibleWithInterpolationError:
+                log.info("Subprotocol format not supported by intperpolation")
+                return
+
+            charge_conversion_factor = (
+                MILLI_TO_BASE_CONVERSION if stim_protocol["stimulation_type"] == "C" else 1
             )
 
-            well_file.force = calculate_force_from_displacement(well_file.displacement)
+            for waveform in stim_sessions_waveforms:
+                if not waveform.shape[-1]:
+                    continue
+                waveform[0] -= wf[TIME_INDICES][0]
+                waveform[1] //= charge_conversion_factor
+                wf.stim_sessions.append(waveform)
 
     def _load_dataframe(self, df: pd.DataFrame) -> None:
-        """Add time and force data to well files in PR.
+        """Add time and force data to well files in PlateRecording.
 
         Args:
             df: pd.Dataframe existing time force data to be added
         """
-        time = df["Time (s)"].values
-        for well in self.wells:
-            well_name = well[WELL_NAME_UUID]
-            well_data = np.vstack((time, df[f"{well_name}__raw"])).astype(np.float64)
-            well.force = well_data
+        force_timepoints = df["Time (s)"].values
+
+        try:
+            stim_timepoints = df["Stim Time (µs)"].values
+        except KeyError:
+            stim_timepoints = None
+
+        for wf in self.wells:
+            well_name = wf[WELL_NAME_UUID]
+            raw_force_amplitudes = df[f"{well_name}__raw"]
+            wf.force = np.vstack((force_timepoints, raw_force_amplitudes)).astype(np.float64)
+
+            if stim_timepoints is None:
+                continue
+
+            stim_col_titles = sorted([col for col in df if f"{well_name}__stim" in col])
+
+            for col_title in stim_col_titles:
+                stim_session_raw = np.array([stim_timepoints, df[col_title]])
+                stim_session = stim_session_raw[:, ~np.isnan(stim_session_raw[1])].astype(int)
+                wf.stim_sessions.append(stim_session)
 
     def write_time_force_csv(self, output_dir: str):
         # get recording name
@@ -436,84 +511,102 @@ class PlateRecording:
 
         force_data = dict({"Time (s)": pd.Series(truncated_time_sec)})
 
-        for well in self.wells:
+        for well in self:
             well_name = well.get(WELL_NAME_UUID, None)
             force_data[well_name] = pd.Series(well.force[1])
 
-        time_force_df = pd.DataFrame(force_data)
-        time_force_df.to_csv(output_path, index=False)
+        time_recording_df = pd.DataFrame(force_data)
+        time_recording_df.to_csv(output_path, index=False)
 
-        return time_force_df, output_path
+        return time_recording_df, output_path
 
-    def to_dataframe(self, from_dataframe: bool = False) -> pd.DataFrame:
-        """
-        Creates DataFrame from PlateRecording with all the data interpolated, normalized, and scaled.
-        The returned dataframe contains one column for time in ms and one column for each well.
+    def to_dataframe(self) -> pd.DataFrame:
+        """Creates DataFrame from PlateRecording with all the data
+        interpolated, normalized, and scaled. The returned dataframe contains
+        one column for time in ms and one column for each well.
 
         The dataframe returned by this method can be used in calls to peak_detector by selecting the
         'time' column and the well column that peak detection should be run on after transposing the
         data, e.g.
 
-        >>> df = to_datafram()
+        >>> df = to_dataframe()
         >>> peak_detector(df[['time', 'A1']].transpose().values)
 
         The dataframe needs to be transposed before converting to NDArray because peak_detector
-        wants an ndarray of shape (2,N) while dataframe[['time', 'A1']] is in shape (N,2)
+        wants an ndarray of shape (2, N) while dataframe[['time', 'A1']] is in shape (N,2)
         """
-        data = {}
+        # get first valid well and set interpolation period. Creating new iter to be safe
+        first_well = next(iter(self))
 
-        # get first valid well and set interpolation period
-        first_well = [pw for pw in self.wells if pw][0]
+        # add interpolated force timepoints
+        if self._created_from_dataframe:
+            raise NotImplementedError("Cannot export a DF if created from a DF. Just use the original")
 
-        if self.is_optical_recording:
-            interp_period = first_well[INTERPOLATED_VALUE_UUID]
-        else:
-            interp_period = INTERPOLATED_DATA_PERIOD_US
+        min_time = min([wf.force[0][0] for wf in self])
+        max_time = max([wf.force[0][-1] for wf in self])
+        interp_period = (
+            first_well[INTERPOLATION_VALUE_UUID] if self.is_optical_recording else INTERPOLATED_DATA_PERIOD_US
+        )
+        interp_timepoints = np.arange(min_time, max_time + interp_period, interp_period)
 
-        if from_dataframe:
-            time_steps = copy.deepcopy(first_well.force[0])
-        else:
-            max_time = max([w.force[0][-1] for w in self.wells if w])
-            time_steps = np.arange(interp_period, max_time + interp_period, interp_period)
+        data = {"Time (s)": pd.Series(interp_timepoints)}
 
-        data["Time (s)"] = pd.Series(time_steps)
+        output_stim_data = first_well.version >= VersionInfo.parse("1.0.0")
 
-        for i, w in enumerate(self.wells):
+        # add stim timepoints
+        if output_stim_data:
+            aggregate_stim_timepoints_us = aggregate_timepoints(
+                [session_data[0] for wf in self for session_data in wf.stim_sessions]
+            )
+            aggregate_stim_timepoints_us_for_plotting = np.repeat(aggregate_stim_timepoints_us, 2)
+            data["Stim Time (µs)"] = pd.Series(aggregate_stim_timepoints_us_for_plotting)
 
-            if not from_dataframe:
-                start_idx, end_idx = truncate(
-                    source_series=time_steps,
-                    lower_bound=w.force[0][0],
-                    upper_bound=w.force[0][-1],
-                )
+        # iterating over self.wells instead of using __iter__ so well_idx is preserved
+        for well_idx, wf in enumerate(self.wells):
+            if not wf:
+                continue
 
-                interp_fn = interpolate.interp1d(w.force[0, :], w.force[1, :])
-                interp_force = interp_fn(time_steps[start_idx : end_idx + 1])
-            else:
-                interp_force = copy.deepcopy(w.force[1, 1:])
+            well_name = wf.get(WELL_NAME_UUID, TWENTY_FOUR_WELL_PLATE.get_well_name_from_well_index(well_idx))
 
-            data[f"{w.get(WELL_NAME_UUID, str(i))}__raw"] = pd.Series(w.force[1, :])
+            # add raw force data
+            data[f"{well_name}__raw"] = pd.Series(wf.force[1, :])
 
-            min_value = min(interp_force)
-            interp_force -= min_value
-            interp_force *= MICRO_TO_BASE_CONVERSION
+            # add unit adjusted + normalized force data
+            start_idx, end_idx = truncate(
+                source_series=interp_timepoints, lower_bound=wf.force[0, 0], upper_bound=wf.force[0, -1]
+            )
 
-            data[w.get(WELL_NAME_UUID, str(i))] = pd.Series(interp_force)
+            interp_fn = interpolate.interp1d(wf.force[0, :], wf.force[1, :])
+            interp_force_unewtons = interp_fn(interp_timepoints[start_idx : end_idx + 1])
+
+            min_value = min(interp_force_unewtons)
+
+            interp_force_newtons_normalized = (interp_force_unewtons - min_value) * MICRO_TO_BASE_CONVERSION
+            data[well_name] = pd.Series(interp_force_newtons_normalized)
+
+            # add stim data
+            if output_stim_data:
+                for i, session_data in enumerate(wf.stim_sessions):
+                    data[f"{well_name}__stim_{i}"] = pd.Series(
+                        realign_interpolated_stim_data(
+                            aggregate_stim_timepoints_us_for_plotting, session_data
+                        )
+                    )
 
         df = pd.DataFrame(data)
-        df = df.dropna()
+        if not output_stim_data:
+            df.dropna(inplace=True)
 
         return df
 
     @staticmethod
     def from_dataframe(path, df: pd.DataFrame):
-        # multi zip files
         # only allowed for one recording at a time assuming a user would only ever pass a dataframe to one recording
         log.info(f"Loading recording from file {os.path.basename(path)}")
-        yield PlateRecording(path, force_df=df)
+        yield PlateRecording(path, recording_df=df)
 
     @staticmethod
-    def from_directory(path):
+    def from_directory(path, **kwargs):
         # multi zip files
         for zf in glob.glob(os.path.join(path, "*.zip"), recursive=True):
             zip_file = zipfile.ZipFile(zf)
@@ -522,20 +615,20 @@ class PlateRecording:
                 with tempfile.TemporaryDirectory() as tmpdir:
                     zip_file.extractall(path=tmpdir)
                     for optical_file in os.scandir(tmpdir):
-                        yield PlateRecording(optical_file.path)
+                        yield PlateRecording(optical_file.path, **kwargs)
                 return
             log.info(f"Loading recording from file {zf}")
-            yield PlateRecording(zf)
+            yield PlateRecording(zf, **kwargs)
 
         # multi optical files
         for of in glob.glob(os.path.join(path, "*.xlsx"), recursive=True):
             log.info(f"Loading optical data from file {of}")
-            yield PlateRecording(of)
+            yield PlateRecording(of, **kwargs)
 
         # directory of .h5 files
         for dir in glob.glob(os.path.join(path, "*"), recursive=True):
             if glob.glob(os.path.join(dir, "*.h5"), recursive=True):
-                yield PlateRecording(dir)
+                yield PlateRecording(dir, **kwargs)
 
     def __iter__(self):
         self._iter = 0
@@ -553,7 +646,12 @@ class PlateRecording:
 
 
 # helpers
-def load_files(path):
+def load_files(
+    path: str, stiffness_factor: Optional[int], inverted_post_magnet_wells: Optional[List[str]] = None
+):
+    if not inverted_post_magnet_wells:
+        inverted_post_magnet_wells = []
+
     h5_files = glob.glob(os.path.join(path, "**", "*.h5"), recursive=True)
 
     recording_files = [f for f in h5_files if "Calibration" not in f]
@@ -564,13 +662,18 @@ def load_files(path):
 
     for f in recording_files:
         log.info(f"Loading data from {os.path.basename(f)}")
-        well_file = WellFile(f)
-        tissue_well_files[well_file[WELL_INDEX_UUID]] = well_file
+        well_name = os.path.splitext(f)[0].split("__")[1]
+        well_file = WellFile(
+            f,
+            stiffness_factor=stiffness_factor,
+            has_inverted_post_magnet=well_name in inverted_post_magnet_wells,
+        )
+        tissue_well_files[well_file[WELL_INDEX_UUID]] = well_file  # type: ignore
 
     for f in calibration_files:
         log.info(f"Loading calibration data from {os.path.basename(f)}")
-        well_file = WellFile(f)
-        baseline_well_files[well_file[WELL_INDEX_UUID]] = well_file
+        well_file = WellFile(f, stiffness_factor=stiffness_factor)
+        baseline_well_files[well_file[WELL_INDEX_UUID]] = well_file  # type: ignore
 
     return tissue_well_files, baseline_well_files
 
