@@ -15,6 +15,7 @@ from .constants import BESSEL_BANDPASS_UUID
 from .constants import BESSEL_LOWPASS_10_UUID
 from .constants import BESSEL_LOWPASS_30_UUID
 from .constants import BUTTERWORTH_LOWPASS_30_UUID
+from .constants import CARDIAC_STIFFNESS_FACTOR
 from .constants import MICRO_TO_BASE_CONVERSION
 from .constants import MILLI_TO_BASE_CONVERSION
 from .constants import MILLIMETERS_PER_MILLITESLA
@@ -225,6 +226,7 @@ def calculate_displacement_from_voltage(
 
 def calculate_force_from_displacement(
     displacement_data: NDArray[(2, Any), np.float64],
+    stiffness_factor: int = CARDIAC_STIFFNESS_FACTOR,
     in_mm: bool = True,
 ) -> NDArray[(2, Any), np.float64]:
     """Convert displacement to force.
@@ -233,17 +235,17 @@ def calculate_force_from_displacement(
 
     Args:
         displacement_data: time and Displacement numpy array. Typically coming from calculate_displacement_from_voltage or the magnet finding alg
-        in_mm: whether this data is in units of mm or not. If coming from calculate_displacement_from_voltage, it is likely in meters and this value should be set to True
+        in_mm: whether this data is in units of mm or not. If coming from calculate_displacement_from_voltage, it is likely in meters and this value should be set to False
 
     Returns:
         A 2D array of time vs Force (Newtons)
     """
-    displacement = displacement_data[1, :]
-    if not in_mm:
-        displacement *= MILLI_TO_BASE_CONVERSION
     time = displacement_data[0, :]
+    displacement = displacement_data[1, :]
+
+    unit_conversion = 1 if in_mm else MILLI_TO_BASE_CONVERSION
 
     # calculate force
-    sample_in_newtons = displacement * NEWTONS_PER_MILLIMETER
+    sample_in_newtons = displacement * unit_conversion * NEWTONS_PER_MILLIMETER * stiffness_factor
 
     return np.vstack((time, sample_in_newtons)).astype(np.float64)
